@@ -1,42 +1,3 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the HebMailPack extension.
- *
- * The Initial Developer of the Original Code is Moofie.
- *
- * Portions created by the Initial Developer are Copyright (C) 2004-2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Eyal Rozenberg <eyalroz@technion.ac.il>
- *   Asaf Romano <mozilla.mano@sent.com>
- *   Ilya Konstantinov <mozilla-code@future.shiny.co.il>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-
 //uncomment the following when this file becomes a module
 //var EXPORTED_SYMBOLS = [ "BiDiMailUI" ];
 
@@ -199,212 +160,294 @@ BiDiMailUI.RegExpStrings.MISDETECTED_RTL_SEQUENCE =
 // E7 E3 22 F9 
 BiDiMailUI.RegExpStrings.CODEPAGE_MISDETECTION_SEQUENCE =
   BiDiMailUI.RegExpStrings.MISDETECTED_RTL_CHARACTER +
-  "{3,}" + "|" +  "(" + "(\\s|\"|\W|^)" +
+  "{3,}" + "|" +  "(" + "(?:\\s|\"|\W|^)" +
   BiDiMailUI.RegExpStrings.MISDETECTED_RTL_CHARACTER +
-  "{2,}(\"|\\s|\W|$)" + ")";
+  "{2,}(?:\"|\\s|\W|$)" + ")";
 
 // TODO: maybe it's better to undecode first, then check whether it's UTF-8;
 // that will probably allow using a char range instead of so many individual
 // chars
 BiDiMailUI.RegExpStrings.MISDETECTED_UTF8_SEQUENCE =
-  // Hebrew
-  "(\\xD7([ \\u017E\\u0152\\u0153\\u02DC\\u2013-\\u2022\\u203A\\u2220\\u2122\\u0090-\\u00BF]|&#65533;) ?\"?){3}" +
-  "|" + 
-  // Arabic
-  "((\\xD8[\\x8C-\\xBF])|(\\xD9[\\x80-\\xB9])|(\\xEF\\xAD[\\x90-\\xBF])|(\\xEF[\\xAE-\\xBA][\\x80-\\xBF])|(\\xEF\\xBB[\\x80-\\xBC])){3}" +
-  "|" + 
-  "\\uFFFD{3,}" +
-  "|" + 
-  "\\u00EF\\u00BB\\u00BF" + // UTF-8 BOM octets
-  "|" +
-  "(\\u05F3[\\u2018-\\u2022\\xA9]){2}";
-   
-// Note: if both doCharset and doUTF8 is false, we only correct HTML entities
-BiDiMailUI.performCorrectiveRecoding = function (
-  element,preferredCharset,mailnewsDecodingType,doCharset,doUTF8) {
-    
-  var needCharsetReapplication = false;
 
+  // Hebrew
+  "(?:" +
+  // just octet sequences...
+  "(?:\\xD6[\xB0-\xBF])|" +
+  "(?:\\xD7[\x80-\xBF])|" +
+  // or windows-1252-ified octet sequences
+  "(?:\\xD7(?:[ \\u017E\\u0152\\u0153\\u02DC\\u2013-\\u2022\\u203A\\u2220\\u2122]|&#65533;))" +
+  " ?\"?){3}" +
+  "|" + 
+
+  // Arabic:
+  "(?:" +
+  // just octet sequences...
+  "(?:\\xD8[\\x8C-\\xBF])|" +
+  "(?:\\xD9[\\x80-\\xB9])|" +
+  "(?:\\xEF\\xAD[\\x90-\\xBF])|" +
+  "(?:\\xEF[\\xAE-\\xBA][\\x80-\\xBF])|" +
+  "(?:\\xEF\\xBB[\\x80-\\xBC])|" +
+  // or windows-1252-ified octet sequences
+  "(?:\\xD9(?:[ \\u20AC\\u0192\\u02C6\\u0160\\u0161\\u0152\\u0153\\u017D\\u017E\\u0178\\u02DC\\u2026\\u2013-\\u203A\\u2122]|&#65533;))" +
+  // ... and at least three characters
+  " ?\"?){3}" +
+  "|" + 
+
+  // while \uFFFD characters are possible when mis-decoding UTF-8 as windows-1252/5/6
+  // (e.g. windows-1255 doesn't have \x81 as a character),
+  // contiguous sequences of them are highly unlikely,
+  // as most relevant Unicode ranges are such that the first UTF-8 byte will be in
+  // the ASCII range and therefore not be decoded as \uFFFD
+  //"|" + 
+  //"\\uFFFD{3,}" +
+  
+  // UTF-8 BOM octets - undecoded
+  // (perhaps we should allow these not just at the beginning of the line?)
+  "^\\u00EF\\u00BB\\u00BF" + 
+  "|" + 
+  // UTF-8 BOM octets - misdecoded as windows-1255 
+  // (for some reason Mozilla gives up on the \xBF -> \u00BF third character)
+  "^\\u05DF\\u00BB[\\u00BF\\uFFFD]" + 
+  "|" + 
+  // Non-breaking space
+  "(?:\\xC2\\xA0)" + 
+  "|" + 
+  // This is heuristic, i.e. you happen to see this sequence here and there
+  "(?:\\u05F3[\\u2018-\\u2022\\xA9]){2}";
+
+// TODO: A botched sequence may differ for windows-1252/5/6; also
+// the FFFDs might be interspersed with single other characters.
+BiDiMailUI.RegExpStrings.BOTCHED_UTF8_DECODING_SEQUENCE =
+  "(?:^|\x0A)\\uFFFD{3}"; 
+   
+BiDiMailUI.performCorrectiveRecoding = function (
+  correctiveRecodingParams) {
 #ifdef DEBUG_performCorrectiveRecoding
-          BiDiMailUI.JSConsoleService.logStringMessage('---------------------------------\nin performCorrectiveRecoding(' + 
-          preferredCharset + ', ' + mailnewsDecodingType + ", " + (doCharset ? "doCharset" : "!doCharset") + ", " + 
-          (doUTF8 ? "doUTF8" : "!doUTF8") + ")");
-          BiDiMailUI.JSConsoleService.logStringMessage('element textContent (all nodes together):\n\n' + element.textContent);
+  BiDiMailUI.JSConsoleService.logStringMessage(
+    '---------------------------------\nin performCorrectiveRecoding(' + 
+    correctiveRecodingParams.preferredCharset + ', ' +
+    correctiveRecodingParams.mailnewsDecodingType + ", " +
+    "correctiveRecodingParams.recodePreferredCharset = " +
+    correctiveRecodingParams.recodePreferredCharset + ", " +
+    "correctiveRecodingParams.recodeUTF8 = " +
+    correctiveRecodingParams.recodeUTF8 + ")");
+    BiDiMailUI.JSConsoleService.logStringMessage('element textContent (all nodes together):\n\n' + element.textContent);
 #endif
-  if (!doCharset && !doUTF8) {
+  if (!correctiveRecodingParams.recodePreferredCharset &&
+      !correctiveRecodingParams.recodeUTF8) {
 #ifdef DEBUG_performCorrectiveRecoding
     BiDiMailUI.JSConsoleService.logStringMessage('nothing to do, returning');
 #endif
     return;
   }
-
-  var codepageMisdetectionExpression =
-    new RegExp (BiDiMailUI.RegExpStrings.CODEPAGE_MISDETECTION_SEQUENCE);
-  var utf8MisdetectionExpression =
-    new RegExp (BiDiMailUI.RegExpStrings.MISDETECTED_UTF8_SEQUENCE);
+  
+  // This redundant setting of the charset is necessary to overcome an
+  // issue with TB 2.x in which the first time you set the charset and
+  // attempted to recode, you'd get a NS_ERROR_FAILURE exception;
+  // see bug 
+  BiDiMailUI.UnicodeConverter.charset = correctiveRecodingParams.preferredCharset;
 
   var treeWalker = document.createTreeWalker(
-    element,
+    correctiveRecodingParams.body,
     NodeFilter.SHOW_TEXT,
     null, // additional filter function
     false
   );
-
   var node;
+  while((node = treeWalker.nextNode())) {
+    if (node.data)
+      node.data = BiDiMailUI.performCorrectiveRecodingOnText(
+        node.data,correctiveRecodingParams);
+  }
+  if (correctiveRecodingParams.recodeUTF8) {
+    correctiveRecodingParams.body.setAttribute('bidimailui-recoded-utf8',true);
+  }
+  if (correctiveRecodingParams.recodePreferredCharset) {
+    correctiveRecodingParams.body.setAttribute('bidimailui-recoded-charset',
+      correctiveRecodingParams.preferredCharset);
+  }
 
-  while((node = treeWalker.nextNode())) {  
-    var lines = node.data.split('\n');
-#ifdef DEBUG_performCorrectiveRecoding
-    BiDiMailUI.JSConsoleService.logStringMessage(
-      "processing text node with " + lines.length + " lines");
+  if (correctiveRecodingParams.subjectSetter) try {
+    correctiveRecodingParams.subjectSetter(
+      BiDiMailUI.performCorrectiveRecodingOnText(
+        correctiveRecodingParams.messageSubject,correctiveRecodingParams));
+  } catch(ex) { }
+  return;
+}
+
+
+BiDiMailUI.codepageMisdetectionExpression =
+  new RegExp (BiDiMailUI.RegExpStrings.CODEPAGE_MISDETECTION_SEQUENCE);
+BiDiMailUI.utf8MisdetectionExpression =
+  new RegExp (BiDiMailUI.RegExpStrings.MISDETECTED_UTF8_SEQUENCE);
+
+
+BiDiMailUI.performCorrectiveRecodingOnText = function(
+  str,correctiveRecodingParams) {
+#ifdef DEBUG_performCorrectiveRecodingOnText
+  BiDiMailUI.JSConsoleService.logStringMessage(
+    "corrective-recording the string:\n" + str);
 #endif
-    for(var i = 0; i < lines.length; i++) {
-      var workingStr; 
+  if (!str) return;
+  if (!correctiveRecodingParams.recodeUTF8 &&
+    !correctiveRecodingParams.recodePreferredCharset) return;
+  var lines = str.split('\n');
+#ifdef DEBUG_performCorrectiveRecodingOnText
+  BiDiMailUI.JSConsoleService.logStringMessage(
+    "string has " + lines.length + " lines");
+#endif
+  for(var i = 0; i < lines.length; i++) {
+    var workingStr; 
 
-#ifdef DEBUG_performCorrectiveRecoding
-      if (doUTF8 && !utf8MisdetectionExpression.test(lines[i])) {
+#ifdef DEBUG_performCorrectiveRecodingOnText
+    if (correctiveRecodingParams.recodeUTF8 &&
+      !BiDiMailUI.utf8MisdetectionExpression.test(lines[i])) {
 #ifdef DEBUG_scancodes
-        BiDiMailUI.JSConsoleService.logStringMessage(
-          "line is not misdecoded UTF-8:\n" + lines[i] +
-          "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
+      BiDiMailUI.JSConsoleService.logStringMessage(
+        "line is not misdecoded UTF-8:\n" + lines[i] +
+        "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
 #else
-        BiDiMailUI.JSConsoleService.logStringMessage(
-          "line is not misdecoded UTF-8:\n" + lines[i]);
+      BiDiMailUI.JSConsoleService.logStringMessage(
+        "line is not misdecoded UTF-8:\n" + lines[i]);
 #endif
-      }
+    }
 #endif
-      // Note: It's _important_ to check for UTF-8 first, because that has the 
-      // much more distinctive [D7-D9] blah [D7-D9] blah [D7-D9] blah pattern!
-      if (doUTF8 && utf8MisdetectionExpression.test(lines[i])) {
-        try {
-          workingStr = lines[i];
-          
-          // at this point, mailnewsDecodingType can only be latin or preferred
-          BiDiMailUI.UnicodeConverter.charset =
-            (mailnewsDecodingType == "latin-charset") ?
-             'windows-1252' : preferredCharset;
+    // Note: It's _important_ to check for UTF-8 first, because that has the 
+    // much more distinctive [D7-D9] blah [D7-D9] blah [D7-D9] blah pattern!
+    if (correctiveRecodingParams.recodeUTF8 &&
+      BiDiMailUI.utf8MisdetectionExpression.test(lines[i])) {
+#ifdef DEBUG_performCorrectiveRecodingOnText
+      BiDiMailUI.JSConsoleService.logStringMessage("trying to recode UTF-8");
+#endif
+      try {
+        workingStr = lines[i];
+        
+        // at this point, correctiveRecodingParams.mailnewsDecodingType can only be latin or preferred
+        BiDiMailUI.UnicodeConverter.charset =
+          (correctiveRecodingParams.mailnewsDecodingType == "latin-charset") ?
+           'windows-1252' : correctiveRecodingParams.preferredCharset;
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage(
-            "decoded as " + BiDiMailUI.UnicodeConverter.charset + ":\n" +
-            workingStr + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "decoded as " + BiDiMailUI.UnicodeConverter.charset + ":\n" +
+          workingStr + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
 #endif
 
         
-          workingStr = BiDiMailUI.UnicodeConverter.ConvertFromUnicode(workingStr);
-          // TODO: not sure we need this next line
-          workingStr += BiDiMailUI.UnicodeConverter.Finish();
+        workingStr = BiDiMailUI.UnicodeConverter.ConvertFromUnicode(workingStr);
+        // TODO: not sure we need this next line
+        workingStr += BiDiMailUI.UnicodeConverter.Finish();
 
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage("undecoded bytes:\n" +
-            workingStr  + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
+        BiDiMailUI.JSConsoleService.logStringMessage("undecoded bytes:\n" +
+          workingStr  + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
 #endif
 
-          // We see a lot of D7 20's instead of D7 A0's which are the 2-byte sequence for 
-          // the Hebrew letter Nun; I guess some clients or maybe even Mozilla replace A0
-          // (a non-breaking space in windows-1252) with 20 (a normal space)
-          workingStr = workingStr.replace(/([\xD7-\xD9])\x20/g, "$1\xA0");
+        // We see a lot of D7 20's instead of D7 A0's which are the 2-byte sequence for 
+        // the Hebrew letter Nun; I guess some clients or maybe even Mozilla replace A0
+        // (a non-breaking space in windows-1252) with 20 (a normal space)
+        workingStr = workingStr.replace(/([\xD7-\xD9])\x20/g, "$1\xA0");
 
-          // remove some higher-than-0x7F characters originating in HTML entities, such as &nbsp;
-          // (we remove them only if they're not the second byte of a two-byte sequence; we ignore
-          // the possibility of their being part of a 3-to-6-byte sequence)
-          workingStr = workingStr.replace(/(^|[\x00-\xBF])\xA0+/g, "$1 ");
+        // remove some higher-than-0x7F characters originating in HTML entities, such as &nbsp;
+        // (we remove them only if they're not the second byte of a two-byte sequence; we ignore
+        // the possibility of their being part of a 3-to-6-byte sequence)
+        workingStr = workingStr.replace(/(^|[\x00-\xBF])\xA0+/g, "$1 ");
 
-          // decode any numeric HTML entities ; weird stuff
-          // will be replaced with the UTF-8 encoding of a \uFFFD (unicode replacement char)
-          workingStr = workingStr.replace(
-            /[\xC2\xDF]*&#(\d+);/g,
-            function() {
-              var res = String.fromCharCode(RegExp.$1);
-              return ((res.charCodeAt(0) > 0xBF) ? "\xEF\xBF\xBD" : res);
-            }
-            );
+        // decode any numeric HTML entities ; 
+        // weird stuff we don't recognize will be replaced with the 
+        // UTF-8 encoding of a \uFFFD (unicode replacement char)
+        workingStr = workingStr.replace(
+          /[\xC2-\xDF]*&#(\d+);/g,
+          function() {
+            var res = String.fromCharCode(RegExp.$1);
+            return ((res.charCodeAt(0) > 0xBF) ? "\xEF\xBF\xBD" : res);
+          }
+          );
 
-          // first byte of a two-byte sequence followed by a byte not completing the sequence
-          workingStr = workingStr.replace(/[\xD7-\xD9]([^\x80-\xBF]|$)/g, "$1");
+        // first byte of a two-byte sequence followed by a byte not completing the sequence
+        workingStr = workingStr.replace(/[\xD7-\xD9]([^\x80-\xBF]|$)/g, "$1");
 
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage(
-            "after preprocessing (decoding of HTML entities, removing NBSPs (A0's)," + 
-            "removing unterminated 2-byte sequences):\n" + workingStr +
-            "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "after preprocessing (decoding of HTML entities, removing NBSPs (A0's)," + 
+          "removing unterminated 2-byte sequences):\n" + workingStr +
+          "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
 #endif
 
-          BiDiMailUI.UnicodeConverter.charset = "UTF-8";
-          workingStr = BiDiMailUI.UnicodeConverter.ConvertToUnicode(workingStr);
+        BiDiMailUI.UnicodeConverter.charset = "UTF-8";
+        workingStr = BiDiMailUI.UnicodeConverter.ConvertToUnicode(workingStr);
           
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage("decoded UTF-8:\n" + workingStr + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "decoded UTF-8:\n" + workingStr + "\n----\n" +
+          BiDiMailUI.JS.stringToScanCodes(lines[i]));
 #endif
-          lines[i] = workingStr;
-        } catch(ex) {
+        lines[i] = workingStr;
+      } catch(ex) {
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage("Exception while trying to recode \n" + lines[i] + "\n\n" + ex);
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "Exception while trying to recode \n" + lines[i] + "\n\n" + ex);
 #else
-          dump("Exception while trying to recode \n" + lines[i] + "\n\n" + ex);
+        dump("Exception while trying to recode \n" + lines[i] + "\n\n" + ex);
 #endif
-          // in some cases we seem to get manged UTF-8 text
-          // which can be fixed by re-applying the current character set to the message,
-          // then recoding if necessary
-          if (/(\x3F[20\x90-\xA8]){3,}/.test(workingStr)) {
-            needCharsetReapplication = true;
-          }
+        // in some cases we seem to get manged UTF-8 text
+        // which can be fixed by re-applying the current character set to the message,
+        // then recoding if necessary; see 
+        // https://www.mozdev.org/bugs/show_bug.cgi?id=18707
+        if (/(\x3F[20\x90-\xA8]){3,}/.test(workingStr)) {
+          correctiveRecodingParams.needCharsetForcing = true;
         }
       }
-      else if (doCharset && codepageMisdetectionExpression.test(lines[i])) {
-        //try{
-          // at this point, mailnewsDecodingType can only be latin or UTF-8
-          BiDiMailUI.UnicodeConverter.charset =
-            (mailnewsDecodingType == "latin-charset") ? 'windows-1252' : "UTF-8";
-#ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage(
-            "decoded as " + BiDiMailUI.UnicodeConverter.charset + ":\n" + lines[i] + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
+    }
+    else if (correctiveRecodingParams.recodePreferredCharset &&
+      BiDiMailUI.codepageMisdetectionExpression.test(lines[i])) {
+#ifdef DEBUG_performCorrectiveRecodingOnText
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "trying to recode preferred charset");
 #endif
-        
-          workingStr = lines[i];
-          workingStr = BiDiMailUI.UnicodeConverter.ConvertFromUnicode(lines[i]);
-          // TODO: not sure we need this next line
-          workingStr += BiDiMailUI.UnicodeConverter.Finish();
-#ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage(
-            "undecoded bytes:\n" + workingStr  + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(workingStr));
+      try {
+        // at this point, correctiveRecodingParams.mailnewsDecodingType can only be latin or UTF-8
+        BiDiMailUI.UnicodeConverter.charset =
+          (correctiveRecodingParams.mailnewsDecodingType == "latin-charset") ? 'windows-1252' : "UTF-8";
+#ifdef DEBUG_performCorrectiveRecodingOnText
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "set charset to" + (correctiveRecodingParams.mailnewsDecodingType == "latin-charset") ? 'windows-1252' : "UTF-8");
 #endif
-          BiDiMailUI.UnicodeConverter.charset = preferredCharset;
-          lines[i] = BiDiMailUI.UnicodeConverter.ConvertToUnicode(workingStr);
 #ifdef DEBUG_scancodes
-          BiDiMailUI.JSConsoleService.logStringMessage(
-            "decoded " + preferredCharset + ":\n" + lines[i] + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "decoded as " + BiDiMailUI.UnicodeConverter.charset + ":\n" + lines[i] +
+          "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
 #endif
-        //} catch(ex) {
+      
+        workingStr = lines[i];
+        workingStr = BiDiMailUI.UnicodeConverter.ConvertFromUnicode(lines[i]);
+        // TODO: not sure we need this next line
+        workingStr += BiDiMailUI.UnicodeConverter.Finish();
 #ifdef DEBUG_scancodes
-        //  BiDiMailUI.JSConsoleService.logStringMessage("Exception while trying to recode \n" + line + "\n\n" + ex);
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "undecoded bytes:\n" + workingStr  + "\n----\n" + 
+          BiDiMailUI.JS.stringToScanCodes(workingStr));
+#endif
+        BiDiMailUI.UnicodeConverter.charset = correctiveRecodingParams.preferredCharset;
+        lines[i] = BiDiMailUI.UnicodeConverter.ConvertToUnicode(workingStr);
+#ifdef DEBUG_scancodes
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "decoded " + correctiveRecodingParams.preferredCharset + ":\n" +
+          lines[i] + "\n----\n" + BiDiMailUI.JS.stringToScanCodes(lines[i]));
+#endif
+      } catch(ex) {
+#ifdef DEBUG_performCorrectiveRecodingOnText
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "Exception while trying to recode \n" + workingStr + "\n\n" + ex);
 #else
-        //  dump("Exception while trying to recode \n" + line + "\n\n" + ex);
+        dump("Exception while trying to recode \n" + workingStr + "\n\n" + ex);
 #endif
-        //}
       }
     }
-#ifdef DEBUG_scancodes
-//    BiDiMailUI.JSConsoleService.logStringMessage("lines:\n");
-//    for(i = 0; i < lines.length; i++) {
-//      BiDiMailUI.JSConsoleService.logStringMessage(lines[i]);
-//    }
-#endif
-    if (doUTF8 || doCharset) {
-      node.data = lines.join('\n');
-    }
-#ifdef DEBUG_scancodes
-//    BiDiMailUI.JSConsoleService.logStringMessage("node.data is now\n" + node.data);
-#endif
   }
-  if (doUTF8) {
-    element.setAttribute('bidimailui-recoded-utf8',true);
-  }
-  if (doCharset) {
-    element.setAttribute('bidimailui-recoded-charset',preferredCharset);
-  }
-  return needCharsetReapplication;
+  return lines.join('\n');
 }
 
+
 BiDiMailUI.matchInText = function(element, expression, matchResults) {
 #ifdef DEBUG_matchInText
   BiDiMailUI.JSConsoleService.logStringMessage(
@@ -474,9 +517,9 @@ BiDiMailUI.directionCheck = function(obj) {
   const RTL_CHARACTER_INNER =
    "\\u0590-\\u05FF\\uFB1D-\\uFB4F\\u0600-\\u06FF\\uFB50-\\uFDFF\\uFE70-\\uFEFC";
   const RTL_CHARACTER = "[" + RTL_CHARACTER_INNER + "]";
-  const RTL_SEQUENCE = "(" +  RTL_CHARACTER + "{2,}|" + RTL_CHARACTER + "\"" +
+  const RTL_SEQUENCE = "(?:" +  RTL_CHARACTER + "{2,}|" + RTL_CHARACTER + "\"" +
            RTL_CHARACTER + ")";
-  const LTR_SEQUENCE = "(" +  "\\w" + "[\\-@\\.']?" + ")" + "{2,}";
+  const LTR_SEQUENCE = "(?:" +  "\\w" + "[\\-@\\.']?" + ")" + "{2,}";
   const NEUTRAL_CHARACTER_INNER =
     " \\f\\r\\t\\v\\u00A0\\u2028\\u2029!-@\[-`\{-\xA0\u2013\\u2014\\uFFFD";
   const NEUTRAL_CHARACTER = "[" + NEUTRAL_CHARACTER_INNER + "]";
@@ -500,19 +543,19 @@ BiDiMailUI.directionCheck = function(obj) {
     "^" + NEUTRAL_CHARACTER_NEW_LINE + "*" + "$");
   var rtlLineExpression = new RegExp (
     // either the text has no non-RTL characters and some RTL characters
-    "(" + "^" + IGNORABLE_CHARACTER_NEW_LINE + "*" + RTL_CHARACTER + IGNORABLE_CHARACTER_NEW_LINE + "*" + "$" + ")" +
+    "(?:" + "^" + IGNORABLE_CHARACTER_NEW_LINE + "*" + RTL_CHARACTER + IGNORABLE_CHARACTER_NEW_LINE + "*" + "$" + ")" +
     "|" +
     // or it has only one non-RTL 'word', with an RTL 'word' before it
-    "(" + "^" + IGNORABLE_CHARACTER + "*" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "+" + LTR_SEQUENCE + IGNORABLE_CHARACTER + "*" +  "$" + ")" +
+    "(?:" + "^" + IGNORABLE_CHARACTER + "*" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "+" + LTR_SEQUENCE + IGNORABLE_CHARACTER + "*" +  "$" + ")" +
     "|" +
     // or it has only one non-RTL 'word', with an RTL 'word' after it
-    "(" + "^" + IGNORABLE_CHARACTER + "*" + LTR_SEQUENCE + IGNORABLE_CHARACTER + "+" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "*" +  "$" + ")" +
+    "(?:" + "^" + IGNORABLE_CHARACTER + "*" + LTR_SEQUENCE + IGNORABLE_CHARACTER + "+" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "*" +  "$" + ")" +
     "|" +
     // or it has a line with two RTL 'words' before any non-RTL characters
-    "(" + "(^|\\n)" + IGNORABLE_CHARACTER + "*" + RTL_SEQUENCE + NEUTRAL_CHARACTER + "+" + RTL_SEQUENCE + ")" +
+    "(?:" + "(^|\\n)" + IGNORABLE_CHARACTER + "*" + RTL_SEQUENCE + NEUTRAL_CHARACTER + "+" + RTL_SEQUENCE + ")" +
     "|" +
     // or it has a line with two RTL 'words' after all non-RTL characters
-    "(" + RTL_SEQUENCE + NEUTRAL_CHARACTER + "+" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "*" + "($|\\n)" + ")" );
+    "(?:" + RTL_SEQUENCE + NEUTRAL_CHARACTER + "+" + RTL_SEQUENCE + IGNORABLE_CHARACTER + "*" + "($|\\n)" + ")" );
 
   if (typeof obj == 'string') {
     if (allNeutralExpression.test(obj)) {
