@@ -1,42 +1,3 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the HebMailPack extension.
- *
- * The Initial Developer of the Original Code is Moofie.
- *
- * Portions created by the Initial Developer are Copyright (C) 2004-2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Eyal Rozenberg <eyalroz@technion.ac.il>
- *   Asaf Romano <mozilla.mano@sent.com>
- *   Ilya Konstantinov <mozilla-code@future.shiny.co.il>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-
 BiDiMailUI.Composition = {
 
   lastWindowToHaveFocus : null,
@@ -46,8 +7,6 @@ BiDiMailUI.Composition = {
     // The default behavior of the Enter key in HTML mail messages
     // is to insert a <br>; the alternative behavior we implement
     // is to close a paragraph and begin a new one
-  paragraphVerticalMargin : null,
-    // Amount of space to add to paragraphs in HTML mail messages
 
   CtrlShiftMachine : {
     // We have implemented a Mealy automaton for implementing the Ctrl+Shift
@@ -94,6 +53,11 @@ BiDiMailUI.Composition = {
         messageHeader:
           (this.messageParams.isReply ? 
           gMsgCompose.originalMsgURI : null),
+        messageSubject: 
+          document.getElementById("msgSubject").value,
+        subjectSetter: function(str) {
+            document.getElementById("msgSubject").value = str;
+          },
         unusableCharsetHandler : function() { return null; },
           //
         needCharsetForcing: false,
@@ -120,6 +84,25 @@ BiDiMailUI.Composition = {
           'avoiding charsetMisdetectionCorrection');
       }
 #endif
+
+      if (IsHTMLEditor()) {
+        var defaultToSendBothTextAndHTML =
+          BiDiMailUI.Prefs.getBoolPref("compose.default_to_send_text_with_html", false);
+        
+        var defaultOptionElementId = 
+          (defaultToSendBothTextAndHTML ? "format_both" : "format_html");
+        document.getElementById(defaultOptionElementId)
+                .setAttribute("checked", "true");
+        OutputFormatMenuSelect(
+          {getAttribute: function () {
+            return defaultOptionElementId;
+          }} );
+
+        BiDiMailUI.Composition.setParagraphMarginsRule();
+        if (BiDiMailUI.Composition.alternativeEnterBehavior)
+          BiDiMailUI.Composition.setParagraphMode();
+      }
+
       BiDiMailUI.Composition.setInitialDirection(this.messageParams);
       clearTimeout(this.workaroundForcingTimeoutId);
     }
@@ -387,6 +370,7 @@ BiDiMailUI.Composition = {
     BiDiMailUI.Prefs.setCharPref("compose.last_used_direction", direction);
   },
 
+  // This inserts any character, actually
   insertControlCharacter : function(controlCharacter) {
     editor = GetCurrentEditor();
     editor.beginTransaction();
@@ -410,7 +394,10 @@ BiDiMailUI.Composition = {
     if (IsHTMLEditor()) {
       var editor = GetCurrentEditor();
       if (!editor) {
-        dump("Could not acquire editor object.");
+#ifdef DEBUG_handleComposeReplyCSS
+        BiDiMailUI.JSConsoleService.logStringMessage('handleComposeReplyCSS failed to acquire editor object.');
+#endif
+        dump('handleComposeReplyCSS failed to acquire editor object.');
         return;
       }
       editor.QueryInterface(nsIEditorStyleSheets);
@@ -418,6 +405,36 @@ BiDiMailUI.Composition = {
     }
   },
 
+  getMessageHead : function() {
+    // assuming the head is the edited document element's first child
+    return document.getElementById("content-frame")
+      .contentDocument.documentElement.firstChild;
+  },
+  
+  addMessageStyleRules : function(styleRulesText) {
+    var editor = GetCurrentEditor();
+    if (!editor) {
+#ifdef DEBUG_addMessageStyleRules
+      BiDiMailUI.JSConsoleService.logStringMessage('addMessageStyleRules failed to acquire editor object.');
+#endif
+      dump('addMessageStyleRules failed to acquire editor object.');
+      return;
+    }
+
+    editor.beginTransaction();
+
+    var css = document.createElement('style');
+    css.type = 'text/css';
+    if (css.styleSheet) {
+      css.styleSheet.cssText = styleRulesText;
+    }
+    else {
+      css.appendChild(document.createTextNode(styleRulesText));
+    }
+    BiDiMailUI.Composition.getMessageHead().appendChild(css);  
+    editor.endTransaction();
+  },
+
   handleDirectionButtons : function() {
     var hiddenButtonsPref =
       !BiDiMailUI.Prefs.getBoolPref("compose.show_direction_buttons", true);
@@ -441,37 +458,33 @@ BiDiMailUI.Composition = {
       .hidden = hiddenButtonsPref;
   },
 
-  loadParagraphMode : function() {
-#ifdef DEBUG_loadParagraphMode
-    BiDiMailUI.JSConsoleService.logStringMessage('loadParagraphMode()');
-#endif
-    // Get the desired space between the paragraphs we add
-    // We use global variables in order to avoid different margins in the same document
-    BiDiMailUI.Composition.paragraphVerticalMargin =
-      BiDiMailUI.Composition.getParagraphMarginFromPref("compose.space_between_paragraphs");
+  setParagraphMarginsRule : function() {
+    // add a style rule to the document with the paragraph
+    // vertical margins dictated by the prefs
+    
+    var margin =
+      BiDiMailUI.Composition.getParagraphMarginFromPrefs();
+    BiDiMailUI.Composition.addMessageStyleRules(
+      "body p { margin-bottom: " + margin + "; margin-top: 0pt; } ");
+  },
 
-    // our extension likes paragraph text entry, not 'body text' - since
-    // paragraph are block elements, with a direction setting
-    try {
-      var editor = GetCurrentEditor();
-      if (editor) {
-        editor.setParagraphFormat("p");
-        // as we don't use doStatefulCommand, we need to update the command
-        // state attribute...
-        document.getElementById("cmd_paragraphState").setAttribute("state", "p");
-        var par = BiDiMailUI.Composition.findClosestBlockElement(editor.selection.focusNode);
-        // Set "Space between paragraphs"
-        par.style.marginBottom = BiDiMailUI.Composition.paragraphVerticalMargin;
-        par.style.marginTop = 0;
-      }
-    } catch(ex) {
-#ifdef DEBUG_loadParagraphMode
-      BiDiMailUI.JSConsoleService.logStringMessage('loadParagraphMode failed:\n' + ex);
+  // Our extension likes "Paragraph Mode" rather than "Body Text" mode
+  // for composing messages, - since paragraph are block elements, with
+  // a direction setting
+
+  setParagraphMode : function() {
+    var editor = GetCurrentEditor();
+    if (!editor) {
+#ifdef DEBUG_handleComposeReplyCSS
+      BiDiMailUI.JSConsoleService.logStringMessage('setParagraphMode  failed to acquire editor object.');
 #endif
-      // since the window is not 'ready', something might throw
-      // an exception here, like inability to focus etc.
-      dump(ex);
+      dump('setParagraphMode failed to acquire editor object.');
+      return;
     }
+    editor.setParagraphFormat("p");
+    // as we don't use doStatefulCommand, we need to update the command
+    // state attribute...
+    document.getElementById("cmd_paragraphState").setAttribute("state", "p");
   },
 
   getDisplayedCopyParams : function(messageURI,messageParams) {
@@ -788,8 +801,8 @@ BiDiMailUI.Composition = {
       // Determine Enter key behavior
       BiDiMailUI.Composition.alternativeEnterBehavior =
         BiDiMailUI.Prefs.getBoolPref("compose.alternative_enter_behavior", true);
-      if (BiDiMailUI.Composition.alternativeEnterBehavior)
-        BiDiMailUI.Composition.loadParagraphMode();
+      // Applying the alternative Enter behavior requires the editor to be
+      // in paragraph mode; but we won't set it until the body is ready
     }
 
     BiDiMailUI.Composition.directionSwitchController.setAllCasters();
@@ -1125,9 +1138,9 @@ BiDiMailUI.Composition = {
         }
         else {
 #ifdef DEBUG_keypress
-    BiDiMailUI.JSConsoleService.logStringMessage('SWITCHING document');
+          BiDiMailUI.JSConsoleService.logStringMessage('SWITCHING document');
 #endif
-    BiDiMailUI.Composition.switchDocumentDirection();
+          BiDiMailUI.Composition.switchDocumentDirection();
         }
         BiDiMailUI.Composition.directionSwitchController.setAllCasters();
         // if Shift has gone up, Ctrl is still down and the next
@@ -1321,7 +1334,8 @@ BiDiMailUI.Composition = {
       return false;
   },
 
-  getParagraphMarginFromPref : function(basePrefName) {
+  getParagraphMarginFromPrefs : function() {
+    var basePrefName = "compose.space_between_paragraphs";
     var marginScale = BiDiMailUI.Prefs.getCharPref(basePrefName + ".scale", "cm");
     var marginVal;
     if (marginScale != "px") {
@@ -1344,6 +1358,9 @@ BiDiMailUI.Composition = {
   insertParagraph : function() {
     var editor = GetCurrentEditor();
     if (!editor) {
+#ifdef DEBUG_insertParagraph
+      BiDiMailUI.JSConsoleService.logStringMessage('Could not acquire editor object.');
+#endif
       dump("Could not acquire editor object.");
       return;
     }
@@ -1423,10 +1440,6 @@ BiDiMailUI.Composition = {
         editor.deleteNode(node);
     }
 
-    // Set "Space between paragraphs"
-    par.style.marginBottom = BiDiMailUI.Composition.paragraphVerticalMargin;
-    par.style.marginTop = 0;
-
     editor.endTransaction();
 
     // ------------------------------- "set old style" ------
@@ -1491,6 +1504,8 @@ BiDiMailUI.Composition.directionSwitchController = {
       case "cmd_switch_paragraph":
       case "cmd_switch_document":
       case "cmd_clear_paragraph_dir":
+      case "cmd_insert_lrm":
+      case "cmd_insert_rlm":
   return true;
       default:
   return false;
@@ -1517,11 +1532,13 @@ BiDiMailUI.Composition.directionSwitchController = {
         retVal = inMessage;
         break;
       case "cmd_switch_document":
+      case "cmd_insert_lrm":
+      case "cmd_insert_rlm":
         retVal = inMessage || inSubjectBox;
         break;
 
       case "cmd_ltr_document":
-        this.setCasterGroup("document");
+        this.setCasterGroup("document",inMessage,inSubjectBox);
       case "cmd_rtl_document":
         retVal = inMessage || inSubjectBox;
         // necessary side-effects performed when
@@ -1530,7 +1547,7 @@ BiDiMailUI.Composition.directionSwitchController = {
 
       case "cmd_ltr_paragraph":
         if (IsHTMLEditor())
-          this.setCasterGroup("paragraph");
+          this.setCasterGroup("paragraph",inMessage,inSubjectBox);
       case "cmd_rtl_paragraph":
         retVal = inMessage;
         // necessary side-effects performed when
@@ -1541,9 +1558,11 @@ BiDiMailUI.Composition.directionSwitchController = {
     return retVal;
   },
 
-  setCasterGroup: function(casterPair) {
+  setCasterGroup: function(casterPair,inMessage,inSubjectBox) {
+#ifdef DEBUG_setCasterGroup
+    BiDiMailUI.JSConsoleService.logStringMessage('setting caster group ' + casterPair);
+#endif
     var casterID, oppositeCasterID, command, direction, commandsAreEnabled;
-    var inMessage = (content == top.document.commandDispatcher.focusedWindow);
 
     // window is not ready to run getComputedStyle before some point,
     // and it would cause a crash if we were to continue (see bug 11712)
@@ -1560,9 +1579,6 @@ BiDiMailUI.Composition.directionSwitchController = {
           document.defaultView
             .getComputedStyle(document.getElementById("content-frame")
             .contentDocument.body, "").getPropertyValue("direction");
-        var inSubjectBox =
-          (document.commandDispatcher.focusedElement ==
-           document.getElementById("msgSubject").inputField);
         commandsAreEnabled = inMessage || inSubjectBox;
         break;
       case "paragraph":
@@ -1598,8 +1614,17 @@ BiDiMailUI.Composition.directionSwitchController = {
   },
 
   setAllCasters: function() {
-    this.setCasterGroup("document");
-    this.setCasterGroup("paragraph");
+#ifdef DEBUG_setAllCasters
+      BiDiMailUI.JSConsoleService.logStringMessage('setting casters.');
+#endif
+    var inMessage = (content == top.document.commandDispatcher.focusedWindow);
+    var inSubjectBox = 
+      (document.commandDispatcher.focusedElement ==
+       document.getElementById("msgSubject").inputField);
+    var retVal = false;
+
+    this.setCasterGroup("document",inMessage,inSubjectBox);
+    this.setCasterGroup("paragraph",inMessage,inSubjectBox);
   },
 
   doCommand: function(command) {
@@ -1625,9 +1650,16 @@ BiDiMailUI.Composition.directionSwitchController = {
       case "cmd_clear_paragraph_dir":
         BiDiMailUI.Composition.clearParagraphDirection();
         break;
+      case "cmd_insert_lrm":
+        BiDiMailUI.Composition.insertControlCharacter('\u200e');
+        break;
+      case "cmd_insert_rlm":
+        BiDiMailUI.Composition.insertControlCharacter('\u200f');
+        break;
+
       default:
         dump("The command \"" + command +
-             "\" isn't supported by the direction switch controller\n");
+             "\" isn't supported by the directionality controller\n");
         return false;
     }
     this.setAllCasters();
