@@ -1,43 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the HebMailPack extension.
- *
- * The Initial Developer of the Original Code is Moofie.
- *
- * Portions created by the Initial Developer are Copyright (C) 2004-2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Eyal Rozenberg <eyalroz@technion.ac.il>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-
 // Code outside BiDi Mail UI should only use the 
 // BiDiMailUI.Display.ActionPhases and perhaps the
-// BiDiMailUI.DisplaysetMessageDirectionForcing function: ActionPhase functions
+// BiDiMailUI.Display.setMessageDirectionForcing function: ActionPhase functions
 // are the four phases of action performed when loading a message
 
 BiDiMailUI.Display = {
@@ -523,22 +486,22 @@ BiDiMailUI.Display = {
   // Detect and attempt to reload/recode content of wholly or partially 
   // mis-decoded messages
   //
-  // Return value: 
-  //   False if the message has been set to be reloaded, True otherwise
   // Notes: 
+  //   When returning, cMCParams.needCharsetForcing indicates
+  //   whether a reload is necessary
+  //
   //   This function assumes the preferred charset is either windows-1255,
   //   windows-1256 or null; see populatePreferredCharset().
-  //   The function does not receive the subject textbox; it derives the
-  //   subject through using the message header and does not recode it;
-  //   see https://www.mozdev.org/bugs/show_bug.cgi?id=22123
   //
   fixLoadedMessageCharsetIssues : function(cMCParams) {
 
     var contentToMatch;
     
-    var messageSubject;
-    if (cMCParams.messageHeader) {
-      messageSubject =
+    // TODO: perhaps we should prefer the undecoded MIME subject over
+    // the decoded one - and decode it ourselves with the charset
+    // to our liking?
+    if (!cMCParams.messageSubject && cMCParams.messageHeader) {
+      cMCParams.messageSubject =
         cMCParams.messageHeader.mime2DecodedSubject;
     }
 
@@ -601,26 +564,25 @@ BiDiMailUI.Display = {
       cMCParams.charsetOverrideInEffect;
 
     // This sets parameter no. 2
-    var mailnewsDecodingType;
 #ifdef DEBUG_fixLoadedMessageCharsetIssues
     BiDiMailUI.JSConsoleService.logStringMessage('current charset used for decoding:\n' + cMCParams.currentCharset);
 #endif
     if ((cMCParams.preferredCharset != null) &&
         (cMCParams.currentCharset == cMCParams.preferredCharset))
-      mailnewsDecodingType = "preferred-charset";
+      cMCParams.mailnewsDecodingType = "preferred-charset";
     else if ((((cMCParams.currentCharset == "ISO-8859-8-I") ||
                (cMCParams.currentCharset == "ISO-8859-8")) && 
               (cMCParams.preferredCharset == "windows-1255") ) ||
              ((cMCParams.currentCharset == "ISO-8859-6") && 
               (cMCParams.preferredCharset == "windows-1255") ) ) {
-      mailnewsDecodingType = "preferred-charset";
+      cMCParams.mailnewsDecodingType = "preferred-charset";
     }
     else switch(cMCParams.currentCharset) {
       case "US-ASCII":
       case "ISO-8859-1":
       case "windows-1252":
       case null:
-        mailnewsDecodingType = "latin-charset"; break;
+        cMCParams.mailnewsDecodingType = "latin-charset"; break;
       case "":
         // sometimes the charset is misread, and Mozilla sets it to "" while
         // using UTF-8; this is the case specifically for
@@ -628,7 +590,7 @@ BiDiMailUI.Display = {
         // in the message... but we can't know that without streaming the raw
         // message, which is expensive
       case "UTF-8":
-        mailnewsDecodingType = "UTF-8"; break;
+        cMCParams.mailnewsDecodingType = "UTF-8"; break;
       default: 
 #ifdef DEBUG_fixLoadedMessageCharsetIssues
     BiDiMailUI.JSConsoleService.logStringMessage(
@@ -637,7 +599,7 @@ BiDiMailUI.Display = {
 #endif
         return true;
     }
-    cMCParams.body.setAttribute('bidimailui-detected-decoding-type',mailnewsDecodingType);
+    cMCParams.body.setAttribute('bidimailui-detected-decoding-type',cMCParams.mailnewsDecodingType);
 
 
     // This sets parameter no. 3 
@@ -645,7 +607,7 @@ BiDiMailUI.Display = {
     var havePreferredCharsetText;
 
     if (cMCParams.preferredCharset != null) {
-      if (mailnewsDecodingType == "preferred-charset") {
+      if (cMCParams.mailnewsDecodingType == "preferred-charset") {
         // text in the preferred charset is properly decoded, so we only
         // need to look for characters in the Hebrew or Arabic Unicode ranges;
         // we look for a sequence, since some odd character may be the result
@@ -658,7 +620,7 @@ BiDiMailUI.Display = {
         // text in the preferred charset is properly decoded, so we only
         // need to look for a character in the Hebrew or Arabic Unicode range
         contentToMatch = new RegExp(
-          (mailnewsDecodingType == "latin-charset") ?
+          (cMCParams.mailnewsDecodingType == "latin-charset") ?
           // Here we want a sequence of Unicode value of characters whose 
           // windows-1252 octet is such that would be decoded as 'clearly'
           // Hebrew or Arabic text; we could be less or more picky depending
@@ -669,7 +631,7 @@ BiDiMailUI.Display = {
           // Latin letters in windows-1252, whose Unicode values are the
           // same as their octets
           "[\\u00C0-\\u00FF]{3,}" :
-          // Here we know that mailnewsDecodingType == "UTF-8"; if
+          // Here we know that cMCParams.mailnewsDecodingType == "UTF-8"; if
           // you decode windows-1255/6 content as UTF-8, you'll get failures
           // because you see multi-octet-starter octets (11xxxxxx) followed
           // by other multi-octet-starter octets rather than 
@@ -681,7 +643,7 @@ BiDiMailUI.Display = {
       }    
       havePreferredCharsetText = 
         BiDiMailUI.matchInText(cMCParams.body, contentToMatch) ||
-        contentToMatch.test(messageSubject);
+        contentToMatch.test(cMCParams.messageSubject);
     }
     else {
       havePreferredCharsetText = false;
@@ -692,13 +654,13 @@ BiDiMailUI.Display = {
     var haveUTF8Text;
     
     contentToMatch = new RegExp (
-      (mailnewsDecodingType == "UTF-8") ?
+      (cMCParams.mailnewsDecodingType == "UTF-8") ?
       // The only characters we can be sure will be properly decoded in windows-1252
       // when they appear after UTF-8 decoding are those with single octets in UTF-8
       // and the same value as windows-1252; if there's anything else we'll be
       // conservative and assume some UTF-8 decoding is necessary
       "[^\\u0000-\\u007F\\u00A0-\\u00FF]" :
-      // mailnewsDecodingType is latin-charset or preferred-charset
+      // cMCParams.mailnewsDecodingType is latin-charset or preferred-charset
       //
       // TODO: some of these are only relevant for UTF-8 misdecoded as windows-1252 
       // (or iso-8859-1; mozilla cheats and uses windows-1252), 
@@ -707,13 +669,13 @@ BiDiMailUI.Display = {
 
     haveUTF8Text = 
       BiDiMailUI.matchInText(cMCParams.body, contentToMatch) ||
-      contentToMatch.test(messageSubject);
+      contentToMatch.test(cMCParams.messageSubject);
 
 #ifdef DEBUG_fixLoadedMessageCharsetIssues
     BiDiMailUI.JSConsoleService.logStringMessage("--------\n " +
       (mustKeepCharset ? "Y" : "N") +
-      ((mailnewsDecodingType == "latin-charset") ? "N" :
-       ((mailnewsDecodingType == "preferred-charset") ? "C" : "U")) +
+      ((cMCParams.mailnewsDecodingType == "latin-charset") ? "N" :
+       ((cMCParams.mailnewsDecodingType == "preferred-charset") ? "C" : "U")) +
       (havePreferredCharsetText ? "Y" : "N") +
       (haveUTF8Text ? "Y" : "N") + 
       "\n--------");
@@ -722,7 +684,7 @@ BiDiMailUI.Display = {
     // ... and now act based on the parameter values
     
     if (!mustKeepCharset) {
-      switch(mailnewsDecodingType) {
+      switch(cMCParams.mailnewsDecodingType) {
         case "latin-charset":
           if (!havePreferredCharsetText) {
             if (!haveUTF8Text) {
@@ -750,16 +712,9 @@ BiDiMailUI.Display = {
             }
             else {
               //NNYY
-              if (BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,true,true)) {
-#ifdef DEBUG_fixLoadedMessageCharsetIssues
-                BiDiMailUI.JSConsoleService.logStringMessage(
-                  "re-applying charset - bug workaround");
-#endif
-                // need to re-apply the same charset, as a workaround for a weird mailnews bug
-                cMCParams.needCharsetForcing = true;
-                cMCParams.charsetToForce = cMCParams.currentCharset;
-                return;
-              }
+              cMCParams.recodeUTF8 = true;
+              cMCParams.recodePreferredCharset = true;
+              // but note we might still need to force the charset!
             }
           }
           break;
@@ -824,71 +779,69 @@ BiDiMailUI.Display = {
           }
       }
     }
-    else { // reloading in different charset is allowed
-      switch(mailnewsDecodingType) {
+    else { // mustKeepCharset
+      switch(cMCParams.mailnewsDecodingType) {
         case "latin-charset":
-          if (!havePreferredCharsetText) {
-            if (!haveUTF8Text) {
-              // YNNN
-            }
-            else {
-              // YNNY
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,false,true);
-            }
-          }
-          else {
-            if (!haveUTF8Text) {
-              // YNYN
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,true,false);
-            }
-            else {
-              // YNYY
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,true,true);
-            }
-          }
+          // YNNN, YNNY, YNYN, YNYY
+          cMCParams.recodePreferredCharset = havePreferredCharsetText;
+          cMCParams.recodeUTF8 = haveUTF8Text;
           break;
         case "preferred-charset":
-          if (!havePreferredCharsetText) {
-            if (!haveUTF8Text) {
-              // YCNN
-            }
-            else {
-              // YCNY
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,false,true);
-            }
-          }
-          else {
-            if (!haveUTF8Text) {
-              // YCYN
-            }
-            else {
-              // YCYY
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,false,true);
-            }
-          }
+          // YCNN, YCNY, YCYN, YCYY
+          cMCParams.recodePreferredCharset = false;
+          cMCParams.recodeUTF8 = haveUTF8Text;
           break;
         case "UTF-8":
-          if (!havePreferredCharsetText) {
-            if (!haveUTF8Text) {
-              // YUNN
-            }
-            else {
-              // YUNY
-            }
-          }
-          else {
-            if (!haveUTF8Text) {
-              // YUYN
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,true,false);
-            }
-            else {
-              // YUYY
-              BiDiMailUI.performCorrectiveRecoding(cMCParams.body,cMCParams.preferredCharset,mailnewsDecodingType,true,false);
-            }
-          }
+          // YUNN, YUNY, YUYN, YUYY
+          cMCParams.recodePreferredCharset = havePreferredCharsetText;
+          cMCParams.recodeUTF8 = false;
       }
     }
-    return true;
+
+    // workaround for bug 23322:
+    // Mozilla may be 'cheating' w.r.t. decoding charset
+    if (!cMCParams.needCharsetForcing) {
+      contentToMatch = new RegExp (
+        BiDiMailUI.RegExpStrings.BOTCHED_UTF8_DECODING_SEQUENCE);
+      if (BiDiMailUI.matchInText(cMCParams.body, contentToMatch) ||
+          contentToMatch.test(cMCParams.messageSubject)) {
+#ifdef DEBUG_fixLoadedMessageCharsetIssues
+          BiDiMailUI.JSConsoleService.logStringMessage(
+            "found a long FFFD sequence (see bug 23322)");
+#endif
+        if (mustKeepCharset) {
+#ifdef DEBUG_fixLoadedMessageCharsetIssues
+          BiDiMailUI.JSConsoleService.logStringMessage(
+            "...but we're not allowed to reload!");
+#endif
+        }
+        else {
+          cMCParams.needCharsetForcing = true;
+          // let's be on the safe side
+          cMCParams.charsetToForce = "windows-1252";
+          return;
+        }
+      }
+    }
+
+    // at this point we _believe_ there's no need for charset forcing,
+    // and we'll only perform corrective recoding
+    if (cMCParams.recodeUTF8 || cMCParams.recodePreferredCharset) {
+      BiDiMailUI.performCorrectiveRecoding(cMCParams);
+      // it may be the case that the corrective recoding suggests we need to force
+      // the charset even though we've already done so; currently this is only
+      // possible in the situation of bug 18707
+      if (!mustKeepCharset && cMCParams.needCharsetForcing) {
+        cMCParams.charsetToForce = cMCParams.currentCharset;
+#ifdef DEBUG_fixLoadedMessageCharsetIssues
+        // need to re-apply the same charset, as a workaround for a weird mailnews bug;
+        // see https://www.mozdev.org/bugs/show_bug.cgi?id=18707
+        BiDiMailUI.JSConsoleService.logStringMessage(
+          "re-applying charset - bug 18707 workaround");
+#endif
+      }
+    }
+    return;
   },
 
 // returns true if numeric entities were found
