@@ -1,45 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the HebMailPack extension.
- *
- * The Initial Developer of the Original Code is Moofie.
- *
- * Portions created by the Initial Developer are Copyright (C) 2004-2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Eyal Rozenberg <eyalroz@technion.ac.il>
- *   Asaf Romano <mozilla.mano@sent.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-
 // This file constains UI and glue code only, calling
 // display logic code elsewhere actually act on the displayed message
 
-
 BiDiMailUI.MessageOverlay = {
   // We set this flag before reloading a message due to 
   // character set mis-detection, to prevent repeated reloading
@@ -62,6 +23,14 @@ BiDiMailUI.MessageOverlay = {
     BiDiMailUI.MessageOverlay.updateDirectionMenuButton(newForcedDirection);
   },
 
+  forceDirection : function(ev,forcedDirection) {
+    var messagePane = document.getElementById("messagepane");
+    var body = messagePane.contentDocument.body;
+    BiDiMailUI.Display.setMessageDirectionForcing(body,forcedDirection);
+    BiDiMailUI.MessageOverlay.updateDirectionMenuButton(forcedDirection);
+    ev.stopPropagation();
+  },
+
   updateDirectionMenuButton : function(forcedDirection) {
 #ifdef DEBUG_updateDirectionMenuButton
     BiDiMailUI.JSConsoleService.logStringMessage(
@@ -138,7 +107,34 @@ BiDiMailUI.MessageOverlay = {
     try {
       msgHdr = gMessageDisplay.displayedMessage;
     } catch(ex) {
-      msgHdr = messenger.msgHdrFromURI(gFolderDisplay.selectedMessageUris[0]);
+      try {
+        msgHdr = messenger.msgHdrFromURI(gFolderDisplay.selectedMessageUris[0]);
+      } catch(ex) {
+        msgHdr = messenger.msgHdrFromURI(GetLoadedMessage());
+      }
+    }
+
+    var displayedMessageSubject;
+    try {
+      // This is a very volatile piece of code; it may very well differ
+      // between TB and SM and between versions, with extensions such as
+      // Mnenhy etc.
+      var expandedSubjectBox = document.getElementById('expandedsubjectBox');
+      if (expandedSubjectBox.hasAttribute("headervalue")) {
+        displayedMessageSubject = expandedSubjectBox.getAttribute("headervalue");
+      } 
+      else {
+        var valueNode = document.getAnonymousElementByAttribute(expandedSubjectBox,"anonid","headerValue");
+        if (valueNode.firstChild) {
+          displayedMessageSubject = valueNode.firstChild.data;
+        }
+        else displayedMessageSubject = valueNode.value;
+      }
+    }
+    catch(ex) {
+#ifdef DEBUG_onLoad
+      BiDiMailUI.JSConsoleService.logStringMessage("couldn't get subject:\n" + ex);
+#endif
     }
 
     var charsetPhaseParams = {
@@ -146,6 +142,13 @@ BiDiMailUI.MessageOverlay = {
       charsetOverrideInEffect: msgWindow.charsetOverride,
       currentCharset: msgWindow.mailCharacterSet,
       messageHeader: msgHdr,
+      messageSubject: displayedMessageSubject,
+      subjectSetter: 
+        function(str) {
+          // using the appropriate setter rather than directly
+          // setting the valueNode's data
+          document.getElementById('expandedsubjectBox').headerValue = str;
+        },
       unusableCharsetHandler : BiDiMailUI.MessageOverlay.promptForDefaultCharsetChange,
       needCharsetForcing: false, // this is an out parameter
       charsetToForce: null       // this is an out parameter
@@ -157,7 +160,8 @@ BiDiMailUI.MessageOverlay = {
       // we're reloading with a different charset, don't do anything else
       return;
     }
-    BiDiMailUI.MessageOverlay.dontReload = false; // clearing BiDiMailUI.MessageOverlay.dontReload for other messages
+    BiDiMailUI.MessageOverlay.dontReload = false; 
+      // clearing BiDiMailUI.MessageOverlay.dontReload for other messages
 
     BiDiMailUI.Display.ActionPhases.htmlNumericEntitiesDecoding(body);
     BiDiMailUI.Display.ActionPhases.quoteBarsCSSFix(domDocument);
